==> pipedrive-2.0.0.tar.gz: pipedrive/src/Plugin/WebformHandler/PipedriveWebformHandler.php <==
<?php

namespace Drupal\pipedrive\Plugin\WebformHandler;

use Drupal\Core\Render\Element;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Pipedrive Webform Handler.
 *
 * @WebformHandler(
 *   id = "pipedrive",
 *   label = @Translation("Pipedrive"),
 *   category = @Translation("External"),
 *   description = @Translation("Sends a webform submission to Pipedrive."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class PipedriveWebformHandler extends WebformHandlerBase {

  /**
   * The Pipedrive service.
   *
   * @var \Drupal\pipedrive\Service\PipedriveServiceInterface
   */
  protected $pipedrive;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->pipedrive = $container->get('pipedrive');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'pipedrive_mapping' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $webform = $this->getWebform();
    // Assemble webform mapping source fields.
    $map_sources = [];
    $elements = $this->webform->getElementsDecoded();
    foreach (Element::children($elements) as $key) {
      if (empty($elements[$key]['#title'])) {
        continue;
      }
      $map_sources[$key] = $elements[$key]['#title'];
    }
    $submission_storage = \Drupal::entityTypeManager()->getStorage('webform_submission');
    $field_definitions = $submission_storage->getFieldDefinitions();
    $field_definitions = $submission_storage->checkFieldDefinitionAccess($webform, $field_definitions);
    foreach ($field_definitions as $key => $field_definition) {
      $map_sources[$key] = $field_definition['title'] . ' (type : ' . $field_definition['type'] . ')';
    }

    $form['pipedrive_mapping'] = [
      '#type' => 'webform_mapping',
      '#title' => $this->t('Webform to Pipedrive mapping'),
      '#description' => $this->t('Only Maps with specified "Pipedrive" will be submitted to Marketo.'),
      '#source__title' => t('Webform Submitted Data'),
      '#destination__title' => t('Pipedrive Field'),
      '#source' => $map_sources,
      '#destination__type' => 'webform_select_other',
      '#destination' => $this->getPipedriveFields(),
      '#default_value' => $this->configuration['pipedrive_mapping'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * Get the fields that can be used for pipedrive.
   *
   * @return array
   *   An array of fields for the used entities.
   */
  protected function getPipedriveFields() {
    $pipedrive_client = $this->pipedrive->getClient();
    $person_fields_controller = $pipedrive_client->getPersonFields();
    $data = $person_fields_controller->getAllPersonFields();
    $person_fields = $this->buildFieldList($data->data, 'person');

    $company_field_controller = $pipedrive_client->getOrganizationFields();
    $data = $company_field_controller->getAllOrganizationFields([]);
    $organization_fields = $this->buildFieldList($data->data, 'organization');

    $deal_field_controller = $pipedrive_client->getDealFields();
    $data = $deal_field_controller->getAllDealFields([]);
    $deal_fields = $this->buildFieldList($data->data, 'deal');

    return array_merge($person_fields, $organization_fields, $deal_fields);
  }

  /**
   * Build a list of field mappings based on a type.
   *
   * @param array $fields
   *   The fields from the api.
   * @param string $type
   *   The type of the object. To add to the mappings.
   *
   * @return array
   *   An array suitable for using in a webform_mapping destination value.
   */
  protected function buildFieldList(array $fields, string $type) {
    $mapped = [];
    foreach ($fields as $field) {
      if ($field->fieldType == "varchar") {
        $label = $type . ' - ' . $field->name;
        if ($field->required) {
          $label = $label . '*';
        }
        $mapped[$type . ':' . $field->key] = $label;
      }
    }
    return $mapped;
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    if ($webform_submission->isCompleted() && $update == FALSE) {
      $mapped_fields = [];
      $data = $webform_submission->getData();

      $user_id = NULL;
      $organization_id = NULL;
      $pipedrive_client = $this->pipedrive->getClient();
      foreach ($this->configuration['pipedrive_mapping'] as $webform_field => $pipedrive_field) {
        [$object, $field] = explode(':', $pipedrive_field);
        $mapped_fields[$object][$field] = $data[$webform_field];
      }

      if (array_key_exists('organization', $mapped_fields)) {
        $organization_controller = $pipedrive_client->getOrganizations();
        try {
          $organization = $organization_controller->addAnOrganization($mapped_fields['organization']);
        }
        catch (\Exception $exception) {
          $logger = $this->getLogger();
          $logger->error('Error saving organization on Pipedrive: @message', ['@message' => $exception->getMessage()]);
        }
        if (isset($organization->data->id)) {
          $organization_id = $organization->data->id;
        }
      }
      if (array_key_exists('person', $mapped_fields)) {
        // If we have an email check to see if the user exists.
        // Only do this on insert I think.
        if ($organization_id) {
          $mapped_fields['person']['org_id'] = $organization_id;
        }

        $person_controller = $pipedrive_client->getPersons();
        try {
          $person = $person_controller->addAPerson($mapped_fields['person']);
        }
        catch (\Exception $exception) {
          $logger = $this->getLogger();
          $logger->error('Error saving user on Pipedrive: @message', ['@message' => $exception->getMessage()]);
        }
        if (isset($person->data->id)) {
          $user_id = $person->data->id;
        }
      }
      if (array_key_exists('deal', $mapped_fields)) {
        if (!$user_id && !$organization_id) {
          $logger = $this->getLogger();
          $logger->error('Data is present for opportunity but no user or organization is present');
        }
        else {
          if ($user_id) {
            $mapped_fields['deal']['person_id'] = $user_id;
          }
          if ($organization_id) {
            $mapped_fields['deal']['org_id'] = $organization_id;
          }
          $deal_controller = $pipedrive_client->getDeals();
          try {
            $deal = $deal_controller->addADeal($mapped_fields['deal']);
          }
          catch (\Exception $exception) {
            $logger = $this->getLogger();
            $logger->error('Error saving deal on Pipedrive: @message', ['@message' => $exception->getMessage()]);
          }
        }
      }
    }
  }

}

==> pipedrive-2.1.0.tar.gz: pipedrive/src/Plugin/WebformHandler/PipedriveWebformHandler.php <==
<?php

namespace Drupal\pipedrive\Plugin\WebformHandler;

use Drupal\Core\Render\Element;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Pipedrive Webform Handler.
 *
 * @WebformHandler(
 *   id = "pipedrive",
 *   label = @Translation("Pipedrive"),
 *   category = @Translation("External"),
 *   description = @Translation("Sends a webform submission to Pipedrive."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class PipedriveWebformHandler extends WebformHandlerBase {

  /**
   * The pipedrive fields service.
   *
   * @var \Drupal\pipedrive\Service\PipedriveFieldsServiceInterface
   */
  protected $pipedriveFields;

  /**
   * The pipedrive service.
   *
   * @var \Drupal\pipedrive\Service\PipedriveSyncServiceInterface
   */
  protected $pipedriveSync;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->pipedriveFields = $container->get('pipedrive.fields');
    $instance->pipedriveSync = $container->get('pipedrive.sync');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'pipedrive_mapping' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $webform = $this->getWebform();
    // Assemble webform mapping source fields.
    $map_sources = [];
    $elements = $this->webform->getElementsDecoded();
    foreach (Element::children($elements) as $key) {
      if (empty($elements[$key]['#title'])) {
        continue;
      }
      $map_sources[$key] = $elements[$key]['#title'];
    }
    $submission_storage = \Drupal::entityTypeManager()->getStorage('webform_submission');
    $field_definitions = $submission_storage->getFieldDefinitions();
    $field_definitions = $submission_storage->checkFieldDefinitionAccess($webform, $field_definitions);
    foreach ($field_definitions as $key => $field_definition) {
      $map_sources[$key] = $field_definition['title'] . ' (type : ' . $field_definition['type'] . ')';
    }

    $form['pipedrive_mapping'] = [
      '#type' => 'webform_mapping',
      '#title' => $this->t('Webform to Pipedrive mapping'),
      '#description' => $this->t('Only Maps with specified "Pipedrive" will be submitted to Marketo.'),
      '#source__title' => t('Webform Submitted Data'),
      '#destination__title' => t('Pipedrive Field'),
      '#source' => $map_sources,
      '#destination__type' => 'webform_select_other',
      '#destination' => $this->pipedriveFields->getPipedriveFields(),
      '#default_value' => $this->configuration['pipedrive_mapping'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    if ($webform_submission->isCompleted() && $update == FALSE) {
      $data = $webform_submission->getData();
      $this->pipedriveSync->syncObject($data, $this->configuration['pipedrive_mapping']);
    }
  }

}
